@@ -5,7 +5,7 @@ bgmm.e.step <- function(X, model.params) {
       if (model.params$d > 1) {
         ss = svd(model.params$cvar[i,,])
         rtas <- ss$d
-        matc = t(ss$u[rtas > 10^-8, ]) %*% diag(rtas[rtas > 10^-8]^(-1/2)) %*% ss$v[rtas > 10^-8,]
+        matc = t(ss$u[rtas > 10^-8, ,drop=FALSE]) %*% diag(rtas[rtas > 10^-8]^(-1/2), nrow = sum(rtas>10^-8)) %*% ss$v[rtas > 10^-8,,drop=FALSE]
         tx = apply(X, 1, get("-"), model.params$mu[i,,drop=F])
         lfik[,i] <-  -colSums((matc %*% tx)^2)/2 - sum(log(2*pi*rtas[rtas > 10^-8]))/2
       } else {
