@@ -6,7 +6,7 @@ soft.e.step <- function(X, model.params) {
       if (model.params$d > 1) {
         ss = svd(model.params$cvar[i,,])
         rtas <- ss$d
-        matc = t(ss$u[rtas > 10^-8, ]) %*% diag(rtas[rtas > 10^-8]^(-1/2)) %*% ss$v[rtas > 10^-8,]
+        matc = t(ss$u[rtas > 10^-8, ,drop=FALSE]) %*% diag(rtas[rtas > 10^-8]^(-1/2), nrow = sum(rtas>10^-8)) %*% ss$v[rtas > 10^-8,,drop=FALSE]
 #        matc = t(ss$u) %*% diag(ss$d^(-1/2)) %*% ss$v
         tx = apply(X, 1, get("-"), model.params$mu[i,,drop=F])
 #        lfik[,i] <-  -colSums((matc %*% tx)^2)/2 - sum(log(2*pi*ss$d))/2
